@@ -58,12 +58,11 @@
 
 #include <math.h>
 
-#define DOUBLE_RGBA_R(x) (double)UINT_RGBA_R(x)/255.0
-#define DOUBLE_RGBA_G(x) (double)UINT_RGBA_G(x)/255.0
-#define DOUBLE_RGBA_B(x) (double)UINT_RGBA_B(x)/255.0
-#define DOUBLE_RGBA_A(x) (double)UINT_RGBA_A(x)/255.0
-
-#define GO_COLOR_TO_CAIRO(x) DOUBLE_RGBA_R(x),DOUBLE_RGBA_G(x),DOUBLE_RGBA_B(x),DOUBLE_RGBA_A(x)
+enum {
+	RENDERER_CAIRO_PROP_0,
+	RENDERER_CAIRO_PROP_CAIRO,
+	RENDERER_CAIRO_PROP_IS_VECTOR,
+};
 
 static void grc_free_marker_data (GogRendererCairo *crend);
 
@@ -104,6 +103,49 @@ gog_renderer_cairo_finalize (GObject *obj)
 	(*parent_klass->finalize) (obj);
 }
 
+static void
+gog_renderer_cairo_set_property (GObject *obj, guint param_id,
+			   GValue const *value, GParamSpec *pspec)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (obj);
+
+	switch (param_id) {
+	case RENDERER_CAIRO_PROP_CAIRO:
+		if (crend->cairo)
+			cairo_destroy (crend->cairo);
+		crend->cairo = g_value_get_pointer (value);
+		if (crend->cairo)
+			cairo_reference (crend->cairo);
+		break;
+
+	case RENDERER_CAIRO_PROP_IS_VECTOR:
+		crend->is_vector = g_value_get_boolean (value);
+		break;
+
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 return; /* NOTE : RETURN */
+	}
+}
+
+static void
+gog_renderer_cairo_get_property (GObject *obj, guint param_id,
+			   GValue *value, GParamSpec *pspec)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (obj);
+
+	switch (param_id) {
+	case RENDERER_CAIRO_PROP_CAIRO:
+		g_value_set_pointer (value, crend->cairo);
+		break;
+	case RENDERER_CAIRO_PROP_IS_VECTOR:
+		g_value_set_boolean (value, crend->is_vector);
+		break;
+
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 break;
+	}
+}
+
 static void
 grc_free_marker_data (GogRendererCairo *crend)
 {
@@ -133,6 +175,10 @@ grc_line_size (GogRenderer const *rend, double width)
 	return floor (width);
 }
 
+/* This is a workaround for a cairo bug, due to its internal
+ * handling of coordinates (16.16 fixed point). */
+#define GO_CAIRO_CLAMP(x) CLAMP((x),-15000,15000)
+
 static void
 grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 {
@@ -141,10 +187,12 @@ grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 			switch (vpath->code) {
 				case ART_MOVETO_OPEN:
 				case ART_MOVETO:
-					cairo_move_to (cr, vpath->x, vpath->y);
+					cairo_move_to (cr, GO_CAIRO_CLAMP (vpath->x), 
+						           GO_CAIRO_CLAMP (vpath->y));
 					break;
 				case ART_LINETO:
-					cairo_line_to (cr, vpath->x, vpath->y);
+					cairo_line_to (cr, GO_CAIRO_CLAMP (vpath->x), 
+						           GO_CAIRO_CLAMP (vpath->y));
 					break;
 				default:
 					break;
@@ -156,16 +204,20 @@ grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 			switch (bpath->code) {
 				case ART_MOVETO_OPEN:
 				case ART_MOVETO:
-					cairo_move_to (cr, bpath->x3, bpath->y3);
+					cairo_move_to (cr, GO_CAIRO_CLAMP (bpath->x3),
+						      	   GO_CAIRO_CLAMP (bpath->y3));
 					break;
 				case ART_LINETO:
-					cairo_line_to (cr, bpath->x3, bpath->y3);
+					cairo_line_to (cr, GO_CAIRO_CLAMP (bpath->x3),
+						           GO_CAIRO_CLAMP (bpath->y3));
 					break;
 				case ART_CURVETO:
-					cairo_curve_to (cr, 
-						       bpath->x1, bpath->y1,
-						       bpath->x2, bpath->y2,
-						       bpath->x3, bpath->y3);
+					cairo_curve_to (cr, GO_CAIRO_CLAMP (bpath->x1), 
+							    GO_CAIRO_CLAMP (bpath->y1),
+							    GO_CAIRO_CLAMP (bpath->x2), 
+							    GO_CAIRO_CLAMP (bpath->y2),
+							    GO_CAIRO_CLAMP (bpath->x3), 
+							    GO_CAIRO_CLAMP (bpath->y3));
 					break;
 				default:
 					break;
@@ -174,37 +226,6 @@ grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 		}
 }
 
-/* Red and blue are inverted in a pixbuf compared to cairo */
-static void
-grc_pixbuf_to_cairo (unsigned char *p, int width, int height, int rowstride)
-{
-	int i,j;
-	unsigned char a;
-	guint t;
-	
-#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
-	
-	for (i = 0; i < height; i++) {
-		for (j = 0; j < width; j++) {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-			MULT(a,    p[2], p[3], t);
-			MULT(p[1], p[1], p[3], t);
-			MULT(p[2], p[0], p[3], t);
-			p[0] = a;
-#else	  
-			a = p[3];
-			MULT(p[3], p[2], a, t);
-			MULT(p[2], p[1], a, t);
-			MULT(p[1], p[0], a, t);
-			p[0] = a;
-#endif
-			p += 4;
-		}
-		p += rowstride - width * 4;
-	}
-#undef MULT
-}
-
 static void
 grc_draw_path (GogRenderer *rend, ArtVpath const *vpath, ArtBpath const*bpath)
 {
@@ -353,12 +374,13 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 	cairo_surface_t *cr_surface = NULL;
 	cairo_matrix_t cr_matrix;
 	GdkPixbuf *pixbuf = NULL;
+	GOImage *image = NULL;
 	GOColor color;
 	double width = grc_line_size (rend, style->outline.width);
 	double x[3], y[3];
-	int i, j, w, h, rowstride;
+	int i, j, w, h;
 	guint8 const *pattern;
-	unsigned char *pixels, *iter;
+	unsigned char *iter;
 
 	g_return_if_fail (bpath != NULL || vpath != NULL);
 
@@ -425,15 +447,8 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 				break;
 			}
 			cairo_fill_extents (cr, &x[0], &y[0], &x[1], &y[1]);
-			pixbuf = gdk_pixbuf_add_alpha (style->fill.image.image, FALSE, 0, 0, 0);
-			pixels = gdk_pixbuf_get_pixels (pixbuf);
-			h = gdk_pixbuf_get_height (pixbuf);
-			w = gdk_pixbuf_get_width (pixbuf);
-			rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-			cr_surface = cairo_image_surface_create_for_data (pixels,
-				CAIRO_FORMAT_ARGB32, w, h, rowstride);
-			grc_pixbuf_to_cairo (pixels, w, h, rowstride);
-			cr_pattern = cairo_pattern_create_for_surface (cr_surface);
+			cr_pattern = go_image_create_cairo_pattern (style->fill.image.image);
+			g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
 			switch (style->fill.image.type) {
 				case GOG_IMAGE_CENTERED:
@@ -484,6 +499,8 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 		cairo_surface_destroy (cr_surface);
 	if (pixbuf)
 		g_object_unref (pixbuf);
+	if (image)
+		g_object_unref (image);
 }
 
 static void
@@ -704,7 +721,7 @@ grc_gsf_gdk_pixbuf_save (const gchar *buf,
 	return ok;
 }
 
-static cairo_status_t
+G_GNUC_UNUSED static cairo_status_t
 grc_cairo_write_func (void *closure,
 		  const unsigned char *data,
 		  unsigned int length)
@@ -726,26 +743,13 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 	GOImageFormatInfo const *format_info;
 	cairo_surface_t *surface = NULL;
 	cairo_status_t status;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *output_pixbuf;
 	double width_in_pts, height_in_pts;
+	gboolean result;
 
 	gog_graph_get_size (renderer->model, &width_in_pts, &height_in_pts);
 
 	switch (format) {
-		case GO_IMAGE_FORMAT_PNG:
-		case GO_IMAGE_FORMAT_JPG:
-			gog_renderer_cairo_update (crend, 
-						   width_in_pts * x_dpi / 72.0, 
-						   height_in_pts * y_dpi / 72.0, 1.0);
-			pixbuf = gog_renderer_cairo_get_pixbuf (crend);
-			if (pixbuf == NULL)
-				return FALSE;
-			format_info = go_image_get_format_info (format);
-			return gdk_pixbuf_save_to_callback (pixbuf,
-							    grc_gsf_gdk_pixbuf_save,
-							    output, format_info->name,
-							    NULL, NULL);
-			break;
 		case GO_IMAGE_FORMAT_PDF:
 		case GO_IMAGE_FORMAT_PS:
 		case GO_IMAGE_FORMAT_SVG:
@@ -756,30 +760,33 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 					surface = cairo_pdf_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo PDF backend missing");
-					return;
+					return FALSE;
 #endif
 				case GO_IMAGE_FORMAT_PS:
 #ifdef CAIRO_HAS_PS_SURFACE
 					surface = cairo_ps_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo PS backend missing");
-					return;
+					return FALSE;
 #endif
 				case GO_IMAGE_FORMAT_SVG:
 #ifdef CAIRO_HAS_SVG_SURFACE
 					surface = cairo_svg_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo SVG backend missing");
-					return;
+					return FALSE;
 #endif
 				default:
 					break;
@@ -808,9 +815,39 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 			return status == CAIRO_STATUS_SUCCESS;
 			break;
 		default:
-			g_warning ("[GogRendererCairo:export_image] unsupported format");
-			return FALSE;
+			format_info = go_image_get_format_info (format);
+			if (!format_info->has_pixbuf_saver) {
+				g_warning ("[GogRendererCairo:export_image] unsupported format");
+				return FALSE;
+			}
+
+			gog_renderer_cairo_update (crend, 
+						   width_in_pts * x_dpi / 72.0, 
+						   height_in_pts * y_dpi / 72.0, 1.0);
+			pixbuf = gog_renderer_cairo_get_pixbuf (crend);
+			if (pixbuf == NULL)
+				return FALSE;
+			format_info = go_image_get_format_info (format);
+			if (!format_info->alpha_support) 
+				output_pixbuf = gdk_pixbuf_composite_color_simple (pixbuf,
+										   gdk_pixbuf_get_width (pixbuf),
+										   gdk_pixbuf_get_height (pixbuf),
+										   GDK_INTERP_NEAREST,
+										   255, 256, 0xffffffff,
+										   0xffffffff);
+
+			else 
+				output_pixbuf = pixbuf;
+			result = gdk_pixbuf_save_to_callback (output_pixbuf,
+							      grc_gsf_gdk_pixbuf_save,
+							      output, format_info->name,
+							      NULL, NULL);
+			if (!format_info->alpha_support)
+				g_object_unref (output_pixbuf);
+			return result;
+			break;
 	}
+	return FALSE;
 }
 
 static void
@@ -820,6 +857,8 @@ gog_renderer_cairo_class_init (GogRendererClass *rend_klass)
 
 	parent_klass = g_type_class_peek_parent (rend_klass);
 	gobject_klass->finalize		= gog_renderer_cairo_finalize;
+	gobject_klass->get_property = gog_renderer_cairo_get_property;
+	gobject_klass->set_property = gog_renderer_cairo_set_property;
 	rend_klass->push_style		= gog_renderer_cairo_push_style;
 	rend_klass->pop_style		= gog_renderer_cairo_pop_style;
 	rend_klass->push_clip  		= gog_renderer_cairo_push_clip;
@@ -834,6 +873,15 @@ gog_renderer_cairo_class_init (GogRendererClass *rend_klass)
 	rend_klass->get_text_OBR	= gog_renderer_cairo_get_text_OBR;
 	rend_klass->line_size		= gog_renderer_cairo_line_size;
 	rend_klass->export_image	= gog_renderer_cairo_export_image;
+
+	g_object_class_install_property (gobject_klass, RENDERER_CAIRO_PROP_CAIRO,
+		g_param_spec_pointer ("cairo", "cairo",
+			"the cairo_t* this renderer uses",
+			G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_klass, RENDERER_CAIRO_PROP_IS_VECTOR,
+		g_param_spec_boolean ("is-vector", "is-vector",
+			"tells if the cairo surface is vectorial or raster", FALSE,
+			G_PARAM_READWRITE));
 }
 
 static void
@@ -865,15 +913,16 @@ gog_renderer_cairo_update (GogRendererCairo *crend, int w, int h, double zoom)
 	GogGraph *graph;
 	GogView *view;
 	GogViewAllocation allocation;
-	cairo_surface_t *surface;
+	GOImage *image = NULL;
 	gboolean redraw = TRUE;
 	gboolean size_changed;
+	gboolean create_cairo = crend->cairo == NULL;
 
 	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (crend), FALSE);
 	g_return_val_if_fail (IS_GOG_VIEW (crend->base.view), FALSE);
 
 	size_changed = crend->w != w || crend->h != h;
-	if (size_changed) {
+	if (size_changed && create_cairo) {
 		if (crend->pixbuf != NULL)
 			g_object_unref (crend->pixbuf);
 		crend->pixbuf = NULL;
@@ -899,14 +948,12 @@ gog_renderer_cairo_update (GogRendererCairo *crend, int w, int h, double zoom)
 	allocation.x = allocation.y = 0.;
 	allocation.w = w;
 	allocation.h = h;
-	
-	surface = cairo_image_surface_create_for_data (gdk_pixbuf_get_pixels (crend->pixbuf),
-						       CAIRO_FORMAT_ARGB32, 
-						       crend->w, crend->h,
-						       gdk_pixbuf_get_rowstride (crend->pixbuf));
-	crend->cairo = cairo_create (surface);
-	cairo_surface_destroy (surface);
-	crend->is_vector = FALSE;
+
+	if (create_cairo) {
+		image = go_image_new_from_pixbuf (crend->pixbuf);
+		crend->cairo = go_image_get_cairo (image);;
+		crend->is_vector = FALSE;
+	}
 
 	if (size_changed) {
 		crend->base.scale_x = w / graph->width;
@@ -929,22 +976,29 @@ gog_renderer_cairo_update (GogRendererCairo *crend, int w, int h, double zoom)
 	redraw |= crend->base.needs_update;
 	crend->base.needs_update = FALSE;
 
-	if (redraw) { 
-		cairo_set_operator (crend->cairo, CAIRO_OPERATOR_CLEAR);
-		cairo_paint (crend->cairo);
+	if (redraw) {
+		if (create_cairo) {
+			/* clear if we created it, not otherwise */
+			cairo_set_operator (crend->cairo, CAIRO_OPERATOR_CLEAR);
+			cairo_paint (crend->cairo);
+		}
 		cairo_set_operator (crend->cairo, CAIRO_OPERATOR_OVER);
 
 		cairo_set_line_join (crend->cairo, CAIRO_LINE_JOIN_ROUND);
 		cairo_set_line_cap (crend->cairo, CAIRO_LINE_CAP_ROUND);
 
 		gog_view_render	(view, NULL);
-		
-		grc_pixbuf_to_cairo (gdk_pixbuf_get_pixels (crend->pixbuf), w, h, 
-				     gdk_pixbuf_get_rowstride (crend->pixbuf));
+
+		if (create_cairo) {
+			go_image_get_pixbuf (image);
+			g_object_unref (image);
+		}
 	}
 
-	cairo_destroy (crend->cairo);
-	crend->cairo = NULL;
+	if (create_cairo) {
+		cairo_destroy (crend->cairo);
+		crend->cairo = NULL;
+	}
 	return redraw;
 }
 
@@ -955,4 +1009,3 @@ gog_renderer_cairo_get_pixbuf (GogRendererCairo *crend)
 
 	return crend->pixbuf;
 }
-
