@@ -38,8 +38,7 @@
 #include <goffice/utils/go-math.h>
 #include <goffice/app/module-plugin-defs.h>
 
-#include <glib/gi18n.h>
-#include <gtk/gtklabel.h>
+#include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
 GOFFICE_PLUGIN_MODULE_HEADER;
@@ -283,9 +282,9 @@ gog_plot1_5d_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	} else if (axis == gog_axis_get_atype (gog_plot1_5d_get_index_axis (model))) {
 		GSList *ptr;
 
-		bounds->val.minima = 0.;
-		bounds->val.maxima = model->num_elements - 1.;
-		bounds->logical.minima = 0.;
+		bounds->val.minima = 1.;
+		bounds->val.maxima = model->num_elements;
+		bounds->logical.minima = 1.;
 		bounds->logical.maxima = go_nan;
 		bounds->is_discrete    = TRUE;
 
@@ -427,7 +426,7 @@ lines_can_add (GogObject const *parent)
 {
 	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
 	return (GOG_PLOT1_5D (series->base.plot)->support_lines &&
-								!series->has_lines);
+		!series->has_lines);
 }
 
 static void
@@ -435,6 +434,8 @@ lines_post_add (GogObject *parent, GogObject *child)
 {
 	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
 	series->has_lines = TRUE;
+	if (GOG_IS_PLOT_DROPBAR (series->base.plot))
+		gog_series_lines_use_markers (GOG_SERIES_LINES (child), TRUE);
 	gog_object_request_update (child);
 }
 
@@ -538,6 +539,8 @@ gog_series1_5d_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef WITH_GTK
+#include <gtk/gtklabel.h>
 static void 
 gog_series1_5d_populate_editor (GogObject *obj,
 				GogEditor *editor,
@@ -560,6 +563,7 @@ gog_series1_5d_populate_editor (GogObject *obj,
 	error_page = gog_error_bar_prefs (series, "errors", horizontal, dalloc, cc);
 	gog_editor_add_page (editor, error_page, _("Error bars"));
 }
+#endif
 
 static void
 gog_series1_5d_finalize (GObject *obj)
@@ -610,7 +614,9 @@ gog_series1_5d_class_init (GogObjectClass *obj_klass)
 	gobject_klass->get_property = gog_series1_5d_get_property;
 	gobject_klass->finalize 	      = gog_series1_5d_finalize;
 	obj_klass->update 	      = gog_series1_5d_update;
+#ifdef WITH_GTK
 	obj_klass->populate_editor    = gog_series1_5d_populate_editor;
+#endif
 	gog_series_klass->dim_changed = gog_series1_5d_dim_changed;
 
 	gog_object_register_roles (obj_klass, roles, G_N_ELEMENTS (roles));
@@ -633,18 +639,9 @@ gog_series1_5d_init (GObject *obj)
 	series->has_lines = FALSE;
 }
 
-static void
-gog_series1_5d_base_class_finalize (GObjectClass *klass)
-{
-	GogObjectClass *go_klass = GOG_OBJECT_CLASS (klass);
-	gog_object_unregister_role (go_klass, "Series lines");
-	gog_object_unregister_role (go_klass, "Drop lines");
-	gog_object_unregister_role (go_klass, "Lines");
-}
-
-GSF_DYNAMIC_CLASS_FULL (GogSeries1_5d, gog_series1_5d,
-	NULL, gog_series1_5d_base_class_finalize, gog_series1_5d_class_init, NULL,
-	gog_series1_5d_init, GOG_SERIES_TYPE, 0, {})
+GSF_DYNAMIC_CLASS (GogSeries1_5d, gog_series1_5d,
+	gog_series1_5d_class_init, gog_series1_5d_init,
+	GOG_SERIES_TYPE)
 
 /* Plugin initialization */
 
@@ -659,6 +656,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_dropbar_plot_register_type (module);
 	gog_dropbar_view_register_type (module);
 	gog_line_series_register_type (module);
+	gog_line_series_view_register_type (module);
 	gog_line_plot_register_type (module);
 	gog_area_plot_register_type (module);
 	gog_line_view_register_type (module);
