@@ -25,18 +25,14 @@
 #include <goffice/utils/go-math.h>
 #include <goffice/utils/go-regression.h>
 #include <glib/gi18n-lib.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkspinbutton.h>
 
 #include <gsf/gsf-impl-utils.h>
 
-static GogObjectClass *gog_polynom_reg_curve_parent_klass;
+#define UNICODE_MINUS_SIGN_C 0x2212
+static int minus_utf8_len;
+static char minus_utf8[6];
 
-static void order_changed_cb (GtkSpinButton *btn, GObject *obj)
-{
-	g_object_set (obj, "dims", gtk_spin_button_get_value_as_int (btn), NULL);
-}
+static GogObjectClass *gog_polynom_reg_curve_parent_klass;
 
 static int
 gog_polynom_reg_curve_build_values (GogLinRegCurve *rc, double const *x_vals,
@@ -93,36 +89,113 @@ gog_polynom_reg_curve_get_value_at (GogRegCurve *curve, double x)
 	return result;
 }
 
+static const char *const exponent[10] = {
+	"\xE2\x81\xB0",
+	"\xC2\xB9",
+	"\xC2\xB2",
+	"\xC2\xB3",
+	"\xE2\x81\xB4",
+	"\xE2\x81\xB5",
+	"\xE2\x81\xB6",
+	"\xE2\x81\xB7",
+	"\xE2\x81\xB8",
+	"\xE2\x81\xB9"
+};
+
+/* Add an exponent, i.e., a super-scripted number.  */
+static void
+append_exponent (GString *res, unsigned int i)
+{
+	if (i >= 10) {
+		append_exponent (res, i / 10);
+		i %= 10;
+	}
+	g_string_append (res, exponent[i]);
+}
+
+
+static void
+append_number (GString *res, double c, gboolean suppress1)
+{
+	size_t prelen = res->len;
+
+	g_string_append_printf (res, "%g", c);
+
+	if (suppress1 && res->len == prelen + 1 && res->str[prelen] == '1')
+		g_string_truncate (res, prelen);
+	else {
+		/* Handle the minuses as in -1.2222e-3.  */
+		size_t i;
+		for (i = prelen; i < res->len; i++)
+			if (res->str[i] == '-') {
+				res->str[i] = minus_utf8[0];
+				g_string_insert_len (res, i + 1, minus_utf8 + 1, minus_utf8_len - 1);
+				i += minus_utf8_len - 1;
+			}
+	}
+}
+
+
 static gchar const*
 gog_polynom_reg_curve_get_equation (GogRegCurve *curve)
 {
 	if (!curve->equation) {
 		GogLinRegCurve *lin = GOG_LIN_REG_CURVE (curve);
-		GString *str = g_string_new ("");
-		int i;
-		if (lin->affine) {
-			if (curve->a[1] > 0.)
-				g_string_printf(str, "y = %g + %g x", curve->a[0], curve->a[1]);
-			else
-				g_string_printf(str, "y = %g - %g x", curve->a[0], -curve->a[1]);
-		} else {
-			g_string_printf(str, "y = %g x", curve->a[1]);
+		GString *str = g_string_new ("y =");
+		int i, lasti = (lin->affine ? 0 : 1), j = 0;
+
+		for (i = lin->dims; i >= lasti; i--) {
+			double c_i = curve->a[i];
+
+			if (c_i == 0.) /* a very rare event */
+				continue;
+
+			/* Break after every three terms present.  */
+			if (j > 0 && j % 3 == 0)
+				g_string_append_c (str, '\n');
+			j++;
+
+			g_string_append_c (str, ' ');
+			if (j != 1) {
+				if (c_i >= 0)
+					g_string_append_c (str, '+');
+				else {
+					g_string_append_len (str, minus_utf8, minus_utf8_len);
+					c_i = -c_i;
+				}				
+				g_string_append_c (str, ' ');
+			}
+
+			append_number (str, c_i, i >= 1);
+
+			if (i >= 1) {
+				g_string_append_c (str, 'x');
+				if (i > 1)
+					append_exponent (str, i);
+			}
 		}
-		for (i = 2; i <= lin->dims; i++) {
-			if (i % 3 == 0)
-				g_string_append (str, "\n");
-			if (curve->a[i] > 0.)
-				g_string_append_printf(str, " + %g x^%d", curve->a[i], i);
-			else
-				g_string_append_printf(str, " - %g x^%d", -curve->a[i], i);
+		if (j == 0) {
+			/* Nothing whatsoever.  */
+			g_string_append (str, " 0");
 		}
+
 		curve->equation = g_string_free (str, FALSE);
 	}
 	return curve->equation;
 }
 
+#ifdef GOFFICE_WITH_GTK
+#include <gtk/gtklabel.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkspinbutton.h>
 static void
-gog_polynom_reg_curve_populate_editor (GogRegCurve *reg_curve, GtkTable *table)
+order_changed_cb (GtkSpinButton *btn, GObject *obj)
+{
+	g_object_set (obj, "dims", gtk_spin_button_get_value_as_int (btn), NULL);
+}
+
+static void
+gog_polynom_reg_curve_populate_editor (GogRegCurve *reg_curve, gpointer table)
 {
 	int rows, columns;
 	GtkWidget *w;
@@ -142,6 +215,7 @@ gog_polynom_reg_curve_populate_editor (GogRegCurve *reg_curve, GtkTable *table)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), lin->dims);
 	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (order_changed_cb), lin);
 }
+#endif
 
 static char const *
 gog_polynom_reg_curve_type_name (G_GNUC_UNUSED GogObject const *item)
@@ -164,9 +238,13 @@ gog_polynom_reg_curve_class_init (GogRegCurveClass *reg_curve_klass)
 
 	reg_curve_klass->get_value_at = gog_polynom_reg_curve_get_value_at;
 	reg_curve_klass->get_equation = gog_polynom_reg_curve_get_equation;
+#ifdef GOFFICE_WITH_GTK
 	reg_curve_klass->populate_editor = gog_polynom_reg_curve_populate_editor;
+#endif
 
 	gog_object_klass->type_name	= gog_polynom_reg_curve_type_name;
+
+	minus_utf8_len = g_unichar_to_utf8 (UNICODE_MINUS_SIGN_C, minus_utf8);
 }
 
 static void
